@@ -31,51 +31,37 @@
 /* CAUTION:  We require that cfb8 and cfb32 were NOT 
 	compiled with CFB_NEED_SCREEN_PRIVATE */
 
-static BSFuncRec cfb8_32BSFuncRec = {
-    cfb8_32SaveAreas,
-    cfb8_32RestoreAreas,
-    (BackingStoreSetClipmaskRgnProcPtr) 0,
-    (BackingStoreGetImagePixmapProcPtr) 0,
-    (BackingStoreGetSpansPixmapProcPtr) 0,
-};
+static DevPrivateKey cfb8_32GCPrivateKey = &cfb8_32GCPrivateKey;
+DevPrivateKey cfb8_32GetGCPrivateKey(void)
+{
+    return cfb8_32GCPrivateKey;
+}
 
-
-int cfb8_32GCPrivateIndex;
-int cfb8_32GetGCPrivateIndex(void) { return cfb8_32GCPrivateIndex; }
-int cfb8_32ScreenPrivateIndex;
-int cfb8_32GetScreenPrivateIndex(void) { return cfb8_32ScreenPrivateIndex; }
-static unsigned long cfb8_32Generation = 0;
+static DevPrivateKey cfb8_32ScreenPrivateKey = &cfb8_32ScreenPrivateKey;
+DevPrivateKey cfb8_32GetScreenPrivateKey(void)
+{
+    return cfb8_32ScreenPrivateKey;
+}
 
 static Bool
 cfb8_32AllocatePrivates(ScreenPtr pScreen)
 {
    cfb8_32ScreenPtr pScreenPriv;
 
-   if(cfb8_32Generation != serverGeneration) {
-	if(((cfb8_32GCPrivateIndex = AllocateGCPrivateIndex()) < 0) ||
-	    ((cfb8_32ScreenPrivateIndex = AllocateScreenPrivateIndex()) < 0))
-	    return FALSE;
-	cfb8_32Generation = serverGeneration;
-   }
-
    if (!(pScreenPriv = xalloc(sizeof(cfb8_32ScreenRec))))
         return FALSE;
 
-   pScreen->devPrivates[cfb8_32ScreenPrivateIndex].ptr = (pointer)pScreenPriv;
+   dixSetPrivate(&pScreen->devPrivates, cfb8_32ScreenPrivateKey, pScreenPriv);
    
    
    /* All cfb will have the same GC and Window private indicies */
-   if(!mfbAllocatePrivates(pScreen,&cfbWindowPrivateIndex, &cfbGCPrivateIndex))
+   if(!mfbAllocatePrivates(pScreen, &cfbGCPrivateKey))
 	return FALSE;
 
-   /* The cfb indicies are the mfb indicies. Reallocating them resizes them */ 
-   if(!AllocateWindowPrivate(pScreen,cfbWindowPrivateIndex,sizeof(cfbPrivWin)))
-	return FALSE;
-
-   if(!AllocateGCPrivate(pScreen, cfbGCPrivateIndex, sizeof(cfbPrivGC)))
+   if(!dixRequestPrivate(cfbGCPrivateKey, sizeof(cfbPrivGC)))
         return FALSE;
 
-   if(!AllocateGCPrivate(pScreen, cfb8_32GCPrivateIndex, sizeof(cfb8_32GCRec)))
+   if(!dixRequestPrivate(cfb8_32GCPrivateKey, sizeof(cfb8_32GCRec)))
         return FALSE;
 
    return TRUE;
@@ -118,8 +104,6 @@ cfb8_32SetupScreen(
     pScreen->ChangeWindowAttributes = cfb8_32ChangeWindowAttributes;
     pScreen->RealizeWindow = cfb32MapWindow;			/* OK */
     pScreen->UnrealizeWindow = cfb32UnmapWindow;		/* OK */
-    pScreen->PaintWindowBackground = cfb8_32PaintWindow;
-    pScreen->PaintWindowBorder = cfb8_32PaintWindow;
     pScreen->CopyWindow = cfb8_32CopyWindow;
     pScreen->CreatePixmap = cfb32CreatePixmap;			/* OK */
     pScreen->DestroyPixmap = cfb32DestroyPixmap; 		/* OK */
@@ -175,7 +159,7 @@ cfb8_32CloseScreen (int i, ScreenPtr pScreen)
 	xfree(pScreenPriv->visualData);
 
     xfree((pointer) pScreenPriv);
-    pScreen->devPrivates[cfb8_32ScreenPrivateIndex].ptr = NULL;
+    dixSetPrivate(&pScreen->devPrivates, cfb8_32ScreenPrivateKey, NULL);
 
     return(cfb32CloseScreen(i, pScreen));
 }
@@ -220,7 +204,6 @@ cfb8_32FinishScreenInit(
 			defaultVisual, nvisuals, visuals))
 	return FALSE;
 
-    pScreen->BackingStoreFuncs = cfb8_32BSFuncRec;
     pScreen->CreateScreenResources = cfb8_32CreateScreenResources;
     pScreen->CloseScreen = cfb8_32CloseScreen;
     pScreen->GetScreenPixmap = cfb32GetScreenPixmap; 	/* OK */
